@@ -4,7 +4,7 @@
 #' the Palm likelihood. This approach was first proposed by Tanaka et
 #' al. (2008) for two-dimensional Thomas processes. Further
 #' generalisations were made by Stevenson, Borchers, and Fewster (in
-#' prep) and Jones-Todd (2017).
+#' revision) and Jones-Todd et al. (in submission).
 #' 
 #' The parameter \code{D} is the density of parent points, which is
 #' always estimated. Possible additional parameters are
@@ -17,7 +17,7 @@
 #'
 #'   \item \code{kappa}, the average length of the surface phase of a
 #'         diving cetacean (when \code{child.dist = "twocamera"}; see
-#'         Stevenson, Borchers, and Fewster, in prep).
+#'         Stevenson, Borchers, and Fewster, in revision).
 #'
 #'   \item \code{sigma}, the standard deviation of dispersion along
 #'         each dimension (when \code{disp} = "gaussian").
@@ -34,18 +34,19 @@
 #' the survey area; (iii) a component named \code{l}, providing the
 #' time lag between cameras (in seconds); and (iv) a component named
 #' \code{tau}, providing the mean dive-cycle duration. See Stevenson,
-#' Borchers, and Fewster (in prep) for details.
+#' Borchers, and Fewster (in revision) for details.
 #'
+#' @references Jones-Todd, C. M., Caie, P., Illian, J., Stevenson,
+#'     B. C., Savage, A., Harrison, D. J., and Bown, J. L. (in
+#'     submission). Identifying unusual structures in tissue sections
+#'     of colon cancer patients using point pattern analysis.
+#' @references Stevenson, B. C., Borchers, D. L., and Fewster,
+#'     R. M. (in revision) Cluster capture-recapture to account for
+#'     identification uncertainty on aerial surveys of animal
+#'     populations.
 #' @references Tanaka, U., Ogata, Y., and Stoyan, D. (2008) Parameter
 #'     estimation and model selection for Neyman-Scott point
 #'     processes. \emph{Biometrical Journal}, \strong{50}: 43--57.
-#' @references Stevenson, B. C., Borchers, D. L., and Fewster,
-#'     R. M. (in prep) Trace-contrast methods to account for
-#'     identification uncertainty on aerial surveys of cetacean
-#'     populations.
-#' @references Jones-Todd, C. M. (2017) \emph{Modelling complex
-#'     dependencies inherent in spatial and spatio-temporal point
-#'     pattern data}. PhD thesis, University of St Andrews.
 #'
 #' @param points A matrix containing locations of observed points,
 #'     where each row corresponds to a point and each column
@@ -92,18 +93,39 @@
 #'
 #' @inheritParams fit.ns
 #' 
-#' @return An R6 reference class object. Extraction of the information
-#'     held within is best handled by functions \link{coef.palm},
-#'     \link{confint.palm}, \link{summary.palm}, and \link{plot.palm}.
+#' @return An R6 reference class object.
+#'
+#' @seealso Use \link{coef.palm} to extract estimated parameters, and
+#'     \link{plot.palm} to plot the estimated Palm intensity
+#'     function. Use \link{boot.palm} to run a parametric bootstrap,
+#'     allowing calculation of standard errors and confidence
+#'     intervals.
+#'
+#' @seealso See \link{sim.ns} to simulate from a Neyman-Scott point
+#'     process.
 #'
 #' @examples
-#' ## Fit model.
+#' ## Fitting model to example data.
 #' fit <- fit.ns(example.2D, lims = rbind(c(0, 1), c(0, 1)), R = 0.5)
-#' ## Print estimates.
+#' ## Printing estimates.
 #' coef(fit)
-#' ## Plot the estimated Palm intensity.
+#' ## Plotting the estimated Palm intensity.
 #' plot(fit)
-#' 
+#' \dontrun{
+#' ## Simulating data and fitting additional models.
+#' set.seed(1234)
+#' ## One-dimensional Thomas process.
+#' data.thomas <- sim.ns(c(D = 10, lambda = 5, sigma = 0.025), lims = rbind(c(0, 1)))
+#' ## Fitting a model to these data.
+#' fit.thomas <- fit.ns(data.thomas$points, lims = rbind(c(0, 1)), R = 0.5)
+#' ## Three-dimensional Matern process.
+#' data.matern <- sim.ns(c(D = 10, lambda = 10, tau = 0.1), disp = "uniform",
+#'                       lims = rbind(c(0, 1), c(0, 2), c(0, 3)))
+#' ## Fitting a model to these data.
+#' fit.matern <- fit.ns(data.matern$points, lims = rbind(c(0, 1), c(0, 2), c(0, 3)),
+#'                      R = 0.5, disp = "uniform")
+#' }
+#'
 #' @export
 fit.ns <- function(points, lims, R, disp = "gaussian", child.dist = "pois", child.info = NULL,
                       sibling.list = NULL, edge.correction = "pbc", start = NULL, bounds = NULL, trace = FALSE){
@@ -113,8 +135,8 @@ fit.ns <- function(points, lims, R, disp = "gaussian", child.dist = "pois", chil
                                                                                 sibling.list = sibling.list),
                                   edge.correction = edge.correction)
     obj <- create.obj(classes = classes.list$classes, points = points, lims = lims, R = R,
-                      child.list = classes.list$child.list, sibling.list = sibling.list, trace = trace,
-                      start = start, bounds = bounds)
+                      child.list = classes.list$child.list, parent.locs = NULL,
+                      sibling.list = sibling.list, trace = trace, start = start, bounds = bounds)
     obj$fit()
     obj
 }
@@ -133,80 +155,96 @@ fit.ns <- function(points, lims, R, disp = "gaussian", child.dist = "pois", chil
 #' the survey area; (iii) a component named \code{l}, providing the
 #' time lag between cameras (in seconds); and (iv) a component named
 #' \code{tau}, providing the mean dive-cycle duration. See Stevenson,
-#' Borchers, and Fewster (in prep) for details.
+#' Borchers, and Fewster (in revision) for details.
 #'
 #' @param pars A named vector containing the values of the parameters
 #'     of the process that generates the points.
 #' 
 #' @inheritParams fit.ns
+#' @param parents An optional matrix containing locations of
+#'     parents. If this is provided, then the parameter \code{D} is
+#'     not required in \code{pars}. If this is not provided, then
+#'     parents are generated from a homogeneous Poisson point process
+#'     with intensity \code{D}.
 #'
 #' @return A list. The first component gives the Cartesian coordinates
-#'     of the generated points. A second component may provide sibling
+#'     of the generated points. The second component returns the
+#'     parent locations. A third component may provide sibling
 #'     information.
 #'
 #' @examples
-#' \dontrun{
-#' set.seed(1234)
-#' ## One-dimensional Thomas process.
+#' ## Simulating from a one-dimensional Thomas process.
 #' data.thomas <- sim.ns(c(D = 10, lambda = 5, sigma = 0.025), lims = rbind(c(0, 1)))
-#' ## Fitting a model to these data.
-#' fit.thomas <- fit.ns(data.thomas$points, lims = rbind(c(0, 1)), R = 0.5)
-#' ## Three-dimensional Matern process.
+#' ## Simulating from a three-dimensional Matern process.
 #' data.matern <- sim.ns(c(D = 10, lambda = 10, tau = 0.1), disp = "uniform",
 #'                       lims = rbind(c(0, 1), c(0, 2), c(0, 3)))
-#' ## Fitting a model to these data.
-#' fit.matern <- fit.ns(data.matern$points, lims = rbind(c(0, 1), c(0, 2), c(0, 3)),
-#'                      R = 0.5, disp = "uniform")
-#' }
 #' 
 #' @export
-sim.ns <- function(pars, lims, disp = "gaussian", child.dist = "pois", child.info = NULL){
+sim.ns <- function(pars, lims, disp = "gaussian", child.dist = "pois", parents = NULL, child.info = NULL){
     classes.list <- setup.classes(fit = FALSE, family = "ns", family.info = list(child.dist = child.dist,
                                                                                  child.info = child.info,
+                                                                                 parent.locs = parents,
                                                                                  disp = disp),
                                   edge.correction = NULL)
     obj <- create.obj(classes = classes.list$classes, points = NULL, lims = lims, R = NULL,
-                      child.list = classes.list$child.list, sibling.list = NULL, trace = NULL,
-                      start = NULL, bounds = NULL)
+                      child.list = classes.list$child.list, parent.locs = classes.list$parent.locs,
+                      sibling.list = NULL, trace = NULL, start = NULL, bounds = NULL)
     obj$simulate(pars)
 }
 
 #' Fitting a model to a void point process
 #'
-#'  Estimates parameters for a void point process by maximising the
+#' Estimates parameters for a void point process by maximising the
 #' Palm likelihood. This approach was first proposed by Tanaka et
 #' al. (2008) for two-dimensional Thomas processes. Generalisation to
-#' d-dimensional void processes was made by Jones-Todd (2017).
+#' d-dimensional void processes was made by Jones-Todd (in
+#' submission).
 #'
 #' Parameters to estimate are as follows:
 #' \itemize{
-#'   \item \code{Dc}, the baseline density of observed points.
+#'   \item \code{Dc}, the baseline density of points prior to the deletion process.
 #'
 #'   \item \code{Dp}, the density of unobserved parents that cause voids.
 #'
 #'   \item \code{tau}, the radius of the deletion process centred at each parent.
 #' }
 #'
+#' @references Jones-Todd, C. M., Caie, P., Illian, J., Stevenson,
+#'     B. C., Savage, A., Harrison, D. J., and Bown, J. L. (in
+#'     submission). Identifying unusual structures in tissue sections
+#'     of colon cancer patients using point pattern analysis.
 #' @references Tanaka, U., Ogata, Y., and Stoyan, D. (2008) Parameter
 #'     estimation and model selection for Neyman-Scott point
 #'     processes. \emph{Biometrical Journal}, \strong{50}: 43--57.
-#' @references Jones-Todd, C. M. (2017) \emph{Modelling complex
-#'     dependencies inherent in spatial and spatio-temporal point
-#'     pattern data}. PhD thesis, University of St Andrews.
 #'
 #' @inheritParams fit.ns
 #'
-#' @return An R6 reference class object. Extraction of the information
-#'     held within is best handled by functions \link{coef.palm},
-#'     \link{confint.palm}, \link{summary.palm}, and \link{plot.palm}.
+#' @return An R6 reference class object.
+#' 
+#' @seealso Use \link{coef.palm} to extract estimated parameters, and
+#'     \link{plot.palm} to plot the estimated Palm intensity
+#'     function. Use \link{boot.palm} to run a parametric bootstrap,
+#'     allowing calculation of standard errors and confidence
+#'     intervals.
 #'
+#' @seealso See \link{sim.void} to simulate from a void process.
+#'
+#' @examples
+#' \dontrun{
+#' set.seed(1234)
+#' ## Simulating a two-dimensional void process.
+#' void.data <- sim.void(c(Dc = 1000, Dp = 10, tau = 0.05), rbind(c(0, 1), c(0, 1)))
+#' ## Fitting model.
+#' fit <- fit.void(void.data$points, rbind(c(0, 1), c(0, 1)), R = 0.5)
+#' }
+#' 
 #' @export
-fit.void <- function(points, lims, R, edge.correction = "pbc", start = NULL, bounds, trace = FALSE){
+fit.void <- function(points, lims, R, edge.correction = "pbc", start = NULL, bounds = NULL, trace = FALSE){
     classes.list <- setup.classes(fit = TRUE, family = "void", family.info = NULL,
                                   edge.correction = edge.correction)
     obj <- create.obj(classes = classes.list$classes, points = points, lims = lims, R = R,
-                      child.list = NULL, sibling.list = NULL, trace = trace, start = start,
-                      bounds = bounds)
+                      child.list = NULL, parent.locs = NULL, sibling.list = NULL,
+                      trace = trace, start = start, bounds = bounds)
     obj$fit()
     obj
 }
@@ -215,16 +253,34 @@ fit.void <- function(points, lims, R, edge.correction = "pbc", start = NULL, bou
 #'
 #' Generates points from a void point process using parameters provided by the user.
 #'
+#' For a list of possible parameter names, see \link{fit.ns}.
+#'
 #' @inheritParams fit.void
 #' @inheritParams sim.ns
+#' @param parents An optional matrix containing locations of
+#'     parents. If this is provided, then the parameter \code{D} is
+#'     not required in \code{pars}. If this is not provided, then
+#'     parents are generated from a homogeneous Poisson point process
+#'     with intensity \code{Dp}.
+#'
+#' @return A list. The first component gives the Cartesian coordinates
+#'     of the generated points. The second component returns the
+#'     parent locations.
+#'
+#' @examples
+#' ## Two-dimensional void process.
+#' void.data <- sim.void(c(Dc = 1000, Dp = 10, tau = 0.05), rbind(c(0, 1), c(0, 1)))
+#' ## Plotting the data.
+#' plot(void.data$points)
+#' points(void.data$parents, pch = 16, col = "red")
 #'
 #' @export
-sim.void <- function(pars, lims){
-    classes.list <- setup.classes(fit = FALSE, family = "void", family.info = NULL,
+sim.void <- function(pars, lims, parents = NULL){
+    classes.list <- setup.classes(fit = FALSE, family = "void", family.info = list(parent.locs = parents),
                                   edge.correction = NULL)
     obj <- create.obj(classes = classes.list$classes, points = NULL, lims = lims, R = NULL,
-                      child.list = NULL, sibling.list = NULL, trace = NULL, start = NULL,
-                      bounds = NULL)
+                      child.list = NULL, parent.locs = classes.list$parent.locs,
+                      sibling.list = NULL, trace = NULL, start = NULL, bounds = NULL)
     obj$simulate(pars)
 }
 
@@ -239,10 +295,12 @@ setup.classes <- function(fit, family, family.info, edge.correction){
     use.binomchild.class <- FALSE
     use.twocamerachild.class <- FALSE
     child.list <- NULL
+    parent.locs <- NULL
     use.thomas.class <- FALSE
     use.matern.class <- FALSE
     use.void.class <- FALSE
     use.totaldeletion.class <- FALSE
+    use.giveparent.class <- FALSE
     ## Sorting out fitting class.
     if (fit){
         use.fit.class <- TRUE
@@ -291,6 +349,11 @@ setup.classes <- function(fit, family, family.info, edge.correction){
         use.void.class <- TRUE
         use.totaldeletion.class <- TRUE
     }
+    ## Sorting out parent location class.
+    if (!is.null(family.info$parent.locs)){
+        use.giveparent.class <- TRUE
+        parent.locs <- family.info$parent.locs
+    }
     classes <- c("fit"[use.fit.class],
                  "pbc"[use.pbc.class],
                  "buffer"[use.buffer.class],
@@ -302,27 +365,34 @@ setup.classes <- function(fit, family, family.info, edge.correction){
                  "thomas"[use.thomas.class],
                  "matern"[use.matern.class],
                  "void"[use.void.class],
-                 "totaldeletion"[use.totaldeletion.class])
-    list(classes = classes, child.list = child.list)
+                 "totaldeletion"[use.totaldeletion.class],
+                 "giveparent"[use.giveparent.class])
+    list(classes = classes, child.list = child.list, parent.locs = parent.locs)
 }
 
 #' Estimation of animal density from two-camera surveys.
 #'
 #' Estimates animal density (amongst other parameters) from two-camera
 #' aerial surveys. This conceptualises sighting locations as a
-#' Neyman-Scott point pattern---estimation is carried out via
-#' \code{fit.ns()}.
+#' Neyman-Scott point pattern.
 #'
 #' This function is simply a wrapper for \code{fit.ns}, and
 #' facilitates the fitting of the model proposed by Stevenson,
-#' Borchers, and Fewster (in prep). This function presents the
+#' Borchers, and Fewster (in revision). This function presents the
 #' parameter \code{D.2D} (two-dimensional cetacean density in
 #' cetaceans per square km) rather than \code{D} for enhanced
 #' interpretability.
 #'
+#' For further details on the cluster capture-recapture estimation
+#' approach, see Fewster, Stevenson and Borchers (2016).
+#'
+#' @references Fewster, R. M., Stevenson, B. C., and Borchers,
+#'     D. L. (2016) Trace-contrast methods for capture-recapture
+#'     without capture histories. \emph{Statistical Science},
+#'     \strong{31}: 245--258.
 #' @references Stevenson, B. C., Borchers, D. L., and Fewster,
-#'     R. M. (in prep) Trace-contrast methods to account for
-#'     identification uncertainty on aerial surveys of cetacean
+#'     R. M. (in revision) Cluster capture-recapture to account for
+#'     identification uncertainty on aerial surveys of animal
 #'     populations.
 #'
 #' @param points A vector (or single-column matrix) containing the
@@ -342,20 +412,27 @@ setup.classes <- function(fit, family, family.info, edge.correction){
 #' @param l The lag between cameras (in seconds).
 #' @param tau Mean dive-cycle duration (in seconds).
 #' @param R Truncation distance (see \link{fit.ns}).
-#' @param edge.correction The method used for the correction of edge
-#'     effects. Either \code{"pbc"} for periodic boundary conditions,
-#'     or \code{"buffer"} for a buffer-zone correction.
-#' @param trace Logical, if \code{TRUE}, parameter values are printed
-#'     to the screen for each iteration of the optimisation procedure.
 #' @inheritParams fit.ns
 #'
-#' @return An R6 reference class object. Extraction of the information
-#'     held within is best handled by functions \link{coef.palm},
-#'     \link{confint.palm}, \link{summary.palm}, and \link{plot.palm}.
+#' @return An R6 reference class object.
+#'
+#' @seealso Use \link{coef.palm} to extract estimated parameters, and
+#'     \link{plot.palm} to plot the estimated Palm intensity
+#'     function. Use \link{boot.palm} to run a parametric bootstrap,
+#'     allowing calculation of standard errors and confidence
+#'     intervals.
+#'
+#' @seealso See \link{sim.twocamera} to simulate sightings from a
+#'     two-camera aerial survey.
 #'
 #' @examples
+#' ## Fitting model.
 #' fit <- fit.twocamera(points = example.twocamera$points, cameras = example.twocamera$cameras,
-#'                     d = 500, w = 0.175, b = 0.5, l = 20, tau = 110, R = 1)
+#'                      d = 500, w = 0.175, b = 0.5, l = 20, tau = 110, R = 1)
+#' ## Printing estimates.
+#' coef(fit)
+#' ## Plotting the estimated Palm intensity.
+#' plot(fit)
 #' 
 #' @export
 fit.twocamera <- function(points, cameras = NULL, d, w, b, l, tau, R,
@@ -366,11 +443,58 @@ fit.twocamera <- function(points, cameras = NULL, d, w, b, l, tau, R,
     } else {
         sibling.list <- siblings.twocamera(cameras)
     }
-    bounds <- list(sigma = c(0, min(R, b/3)))
+    if (is.null(bounds)){
+        bounds <- list(sigma = c(0, min(R, b/3)))
+    } else if (!any(names(bounds) == "sigma")){
+        bounds[["sigma"]] <- c(0, min(R, b/3))
+    }
     fit.ns(points = points, lims = rbind(c(0, d)), R = R,
-              child.dist = "twocamera",
-              child.info = list(w = w, b = b, l = l, tau = tau),
-              sibling.list = sibling.list, start = start, bounds = bounds, trace = trace)
+           child.dist = "twocamera",
+           child.info = list(w = w, b = b, l = l, tau = tau),
+           sibling.list = sibling.list, edge.correction = edge.correction,
+           start = start, bounds = bounds, trace = trace)
+}
+
+#' Simulating data from two-camera aerial surveys.
+#'
+#' @param pars A vector containing elements named \code{D.2D},
+#'     \code{kappa}, and \code{sigma}, providing values of animal
+#'     density (animals per square km), average duration of surface
+#'     phase (s), and dispersion (km).
+#' @param parents An optional vector containing the parent locations
+#'     for all animals within the area of interest, given in distance
+#'     along the transect (in km). If this is provided, then the
+#'     parameter \code{D.2D} is not required in \code{pars}. If this
+#'     is not provided, then parent locations are generated from a
+#'     homogeneous Poisson point process with intensity \code{D.2D}.
+#' @inheritParams fit.twocamera
+#'
+#' @return A list. The first component gives the distance along the
+#'     transect of detected individuals. The second gives the parent
+#'     locations. The third identifies which parent location generated
+#'     each detected individual. The fourth gives the distance from
+#'     the transect centre line of the detection location. The fifth
+#'     provides observed sibling information.
+#'
+#' @examples
+#' twocamera.data <- sim.twocamera(c(D.2D = 1.3, kappa = 27, sigma = 0.02), d = 500,
+#'                                 w = 0.175, b = 0.5, l = 20, tau = 110)
+#'
+#' @export
+sim.twocamera <- function(pars, d, w, b, l, tau, parents = NULL){
+    if (!is.null(parents)){
+        parents <- matrix(parents, ncol = 1)
+    }
+    family.info <- list(child.dist = "twocamera",
+                        child.info = list(w = w, b = b, l = l, tau = tau),
+                        parent.locs = parents, disp = "gaussian")
+    classes.list <- setup.classes(fit = FALSE, family = "ns",
+                                  family.info = family.info)
+    obj <- create.obj(classes = classes.list$classes, points = NULL, lims = rbind(c(0, d)),
+                      R = NULL, child.list = classes.list$child.list,
+                      parent.locs = classes.list$parent.locs, sibling.list = NULL,
+                      trace = NULL, bounds = NULL)
+    obj$simulate(pars)
 }
 
 #' Bootstrapping for fitted models
@@ -379,17 +503,16 @@ fit.twocamera <- function(points, cameras = NULL, d, w, b, l, tau, R,
 #' using the \code{palm} package.
 #'
 #' @return The original model object containing additional information
-#'     from the boostrap procedure. These are accessed by functions
-#'     such as \link{summary}.
+#'     from the bootstrap procedure. These are accessed by functions
+#'     such as \link{summary.palm} and \link{confint.palm}. The
+#'     bootstrap parameter estimates can be found in the \code{boots}
+#'     component of the returned object.
 #'
 #' @param fit A fitted object.
 #' @param N The number of bootstrap resamples.
 #' @param prog Logical, if \code{TRUE}, a progress bar is printed to
 #'     the console.
 #'
-#' @return The original R6 reference object, with additional bootstrap
-#'     information attached.
-#'
 #' @examples
 #' ## Fit model.
 #' fit <- fit.ns(example.2D, lims = rbind(c(0, 1), c(0, 1)), R = 0.5)
@@ -409,50 +532,3 @@ boot.palm <- function(fit, N, prog = TRUE){
 
 
 
-## Roxygen code for NAMESPACE.
-#' @import methods Rcpp R6
-#' @importFrom graphics abline axis box lines par plot.new plot.window title
-#' @importFrom gsl hyperg_2F1
-#' @importFrom mvtnorm rmvnorm
-#' @importFrom spatstat crossdist
-#' @importFrom stats coef dist integrate nlminb pbeta pgamma pnorm printCoefmat qnorm quantile rbinom rpois runif sd
-#' @importFrom utils setTxtProgressBar txtProgressBar
-#' @useDynLib palm
-NULL
-
-## Data documentation.
-
-#' 1-dimensional example data
-#'
-#' Simulated data, with children points generated from a Binomial(4,
-#' 0.5) distribution.
-#'
-#' @name example.1D
-#' @format A matrix.
-#' @usage example.1D
-#' @docType data
-#' @keywords datasets
-NULL
-
-#' 2-dimensional example data
-#'
-#' Simulated data, with children points generated from a Binomial(2,
-#' 0.5) distribution.
-#'
-#' @name example.2D
-#' @format A matrix.
-#' @usage example.2D
-#' @docType data
-#' @keywords datasets
-NULL
-
-#' Two-camera example data.
-#'
-#' Simulated data from a two-camera aerial survey.
-#'
-#' @name example.twocamera
-#' @format A list.
-#' @usage example.twocamera
-#' @docType data
-#' @keywords datasets
-NULL
